package landlordApp;

import java.util.ArrayList;
import java.util.List;

/**
 * -Pitää yllä asukkaiden rekisteriä, osaa lisätä ja
 * poistaa asukkaan                                
 * -Lukee ja kirjoittaa asukkaat tiedostoon         
 * -Osaa etsiä ja lajitella                         
 * @author Nico & Eelis
 * @version 12.3.2020
 *
 */
public class Asukkaat {
	 private final ArrayList<Asukas> asukkaat = new ArrayList<Asukas>();
	 
	 
	 /**
	  * @return asukkaiden lukumäärä
	  */
	 public int getLkm() {
		 return asukkaat.size();
	 }
	 
	 /**
	  * Lisää asukkaiden listaan uuden asukkaan.
	  * @param asukas lisättävä asukas
	  * @example
	  * <pre name="test">
	  * Asukkaat asukkaat = new Asukkaat();
	  * Asukas asukas1 = new Asukas(1); Asukas asukas2 = new Asukas(1);
	  * asukkaat.getLkm() === 0;
	  * asukkaat.lisaa(asukas1); asukkaat.getLkm() === 1;
	  * asukkaat.lisaa(asukas2); asukkaat.getLkm() === 2;
	  * asukkaat.lisaa(asukas2); asukkaat.getLkm() === 3;
	  * asukkaat.anna(0) === asukas1;
	  * asukkaat.anna(1) === asukas2;
	  * asukkaat.anna(2) === asukas2;
	  * asukkaat.anna(1) == asukas1 === false;
	  * asukkaat.anna(1) == asukas2 === true;
	  * asukkaat.anna(3) === asukas1; #THROWS IndexOutOfBoundsException
	  * asukkaat.lisaa(asukas1); asukkaat.getLkm() === 4;
	  * asukkaat.lisaa(asukas1); asukkaat.getLkm() === 5;
	  * asukkaat.lisaa(asukas1);
	  * </pre>
	  */
	 public void lisaa(Asukas asukas) {
		 this.asukkaat.add(asukas);
	 }
	 
	 /**
	  * Palauttaa paikassa i olevan asukkaan viitteen
	  * @param i indeksi arraylistissä
	  * @return etsitty asukas
	  * @throws IndexOutOfBoundsException jos indeksi on virheellinen, eli liian suuri tai pieni.
	  */
	 public Asukas anna(int i) throws IndexOutOfBoundsException {
		 if(i < 0 || asukkaat.size() <= i)
	            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
	        return asukkaat.get(i);
	 }
	 
	 /**
	  * Palauttaa listan asukkaista, joilla on täsmäävä asuntoId.
	  * @param id asunnon id
	  * @return lista löydetyistä asukkaista
	  * @example
	  * #import java.util.List;
	  * <pre name="test">
	  * Asukkaat asukkaat = new Asukkaat();
	  * Asukas asukas1 = new Asukas(1);
	  * Asukas asukas2 = new Asukas(2);
	  * Asukas asukas3 = new Asukas(1);
	  * asukkaat.lisaa(asukas1);
	  * asukkaat.lisaa(asukas2);
	  * asukkaat.lisaa(asukas3);
	  * List<Asukas> loydetyt1 = asukkaat.annaAsukkaat(1);
	  * loydetyt1.size() === 2;
	  * List<Asukas> loydetyt2 = asukkaat.annaAsukkaat(2);
	  * loydetyt2.size() === 1;
	  * loydetyt1.get(0) === asukas1;
	  * </pre>
	  */
	 public List<Asukas> annaAsukkaat(int id){
	        List<Asukas> loydetyt = new ArrayList<Asukas>();
	        for (Asukas as : asukkaat)
	            if(as.getAsuntoId() == id) loydetyt.add(as);
	        
	        return loydetyt;
	        
	    }
	 
	 /**
	  * @param args ei käytössä
	  */
	public static void main(String[] args) {
		Asukkaat asukkaat = new Asukkaat();
		Asukas a1 = new Asukas(1);
		Asukas a2 = new Asukas(1);
		
		a1.rekisteroi();
		a1.taytaOletus();
		a2.rekisteroi();
		a2.taytaOletus();
		
		asukkaat.lisaa(a1);
		asukkaat.lisaa(a2);
		asukkaat.lisaa(a2);
		asukkaat.lisaa(a2);
		asukkaat.lisaa(a2);
		
		System.out.println("=============== Asukkaat testi ===============");
		
		for (int i = 0; i < asukkaat.getLkm(); i++) {
			Asukas asukas = asukkaat.anna(i);
			System.out.println("Asukkaan indeksi: " + i);
			asukas.tulosta(System.out);
		}
		
	}
}
