package landlordApp;
import java.util.ArrayList; // import the ArrayList class

/**
 * Vastuualueet:
 * -Pitää yllä asuntojen rekisteriä, osaa lisätä ja poistaa asunnon
 * -Lukee ja kirjoittaa asunnot tiedostoon
 * -Osaa etsiä ja lajitella
 * Avustajat: 
 * Asunto
 * @author Nico & Eelis
 * @version 12.3.2020
 */
public class Asunnot {
    
    private static final int MAX_LKM = 5;
    private final ArrayList<Asunto> asunnot = new ArrayList<Asunto>();
    
    /**
     * Palauttaa asuntojen lukumäärän
     * @return asuntojen lukumäärä
     */
    public int getLkm() {
        return asunnot.size();
    }
    
    /**
     * Lisää asunnon asuntojen listaan.
     * @param asunto lisättävä asunto
     * @throws SailoException jos liikaa asuntoja
     * @example
	 * <pre name="test">
	 * #THROWS SailoException
	 * Asunnot asunnot = new Asunnot();
	 * Asunto asunto1 = new Asunto(1); Asunto asunto2 = new Asunto(1);
	 * asunnot.getLkm() === 0;
	 * asunnot.lisaa(asunto1); asunnot.getLkm() === 1;
	 * asunnot.lisaa(asunto2); asunnot.getLkm() === 2;
	 * asunnot.lisaa(asunto2); asunnot.getLkm() === 3;
	 * asunnot.anna(0) === asunto1;
	 * asunnot.anna(1) === asunto2;
	 * asunnot.anna(2) === asunto2;
	 * asunnot.anna(1) == asunto1 === false;
	 * asunnot.anna(1) == asunto2 === true;
	 * asunnot.anna(3) === asunto1; #THROWS IndexOutOfBoundsException
	 * asunnot.lisaa(asunto1); asunnot.getLkm() === 4;
	 * asunnot.lisaa(asunto1); asunnot.getLkm() === 5;
	 * asunnot.lisaa(asunto1); #THROWS SailoException
	 * </pre>
     */
    public void lisaa(Asunto asunto) throws SailoException {
        if (MAX_LKM <= asunnot.size()) throw new SailoException("Liikaa asuntoja.");
        asunnot.add(asunto);
    }
    
    /**
     * Palauttaa paikassa i olevan asunnon
     * @param i asunnon indeksi
     * @return asunto
     * @throws IndexOutOfBoundsException jos paikassa i ei asuntoa
     */
    public Asunto anna(int i) throws IndexOutOfBoundsException {
        if(i < 0 || asunnot.size() <= i)
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return asunnot.get(i);
    }
    
    /**
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Asunnot talot = new Asunnot();
        Asunto talo1 = new Asunto(1); Asunto talo2 = new Asunto(1);
        
        talo1.rekisteroi();
        talo2.rekisteroi();
        talo1.taytaOletus();
        talo2.taytaOletus();
        
        try {
            talot.lisaa(talo1);
            talot.lisaa(talo2);
            talot.lisaa(talo2);
            talot.lisaa(talo2);
            talot.lisaa(talo2);
            talot.lisaa(talo2);
        }
        catch (SailoException ex){
            System.err.println(ex.getMessage());
        }
        
        System.out.println("=============== Talot testi ===============");
        for (int i = 0; i < talot.getLkm(); i++) {
            Asunto talo = talot.anna(i);
            System.out.println("Talon indeksi: " + i);
            talo.tulosta(System.out);
        }
    }
}
