/**
 * 
 */
package landlordApp;

import java.util.List;

/**
 * Vastuualueet:
 * -Huolehtii Talot, Asukkaat ja Asunnot välisestä
 *  yhteistyöstä ja välittää tietoja näiden välillä pyydettäessä.
 * -Lukee ja kirjoittaa landlordin tiedostoon
 *  pyytämällä apua avustajilta
 *  Avustajat:
 *  Talo
 *  Talot
 *  Asukas
 *  Asukkaat
 *  Asunto
 *  Asunnot
 * @author Nico & Eelis
 * @version 12.3.2020
 */
public class Landlord {
	
	 private final Talot talot = new Talot();
	 private final Asukkaat asukkaat = new Asukkaat();
	 private final Asunnot asunnot = new Asunnot();
	
	/**
	 * Palauttaa talojen lukumäärän
	 * @return talojen lukumäärä
	 */
	public int getTaloja() {
		return talot.getLkm();
	}
	
	
	/**
     * Palauttaa asuntojen lukumäärän
     * @return asuntojen lukumäärä
     */
	public int getAsuntoja() {
	   return asunnot.getLkm();
	}
	
	/**
	 * Palauttaa asuntojen lukumäärän
	 * @return asuntojen lukumäärä
	 */
	public int getAsukkaita() {
		return asukkaat.getLkm();
	}
	
	/**
	 * Lisää talon tietokantaan.
	 * @param talo talo, joka halutaan lisätä tietokantaan.
	 * @throws SailoException jos on liikaa taloja
	 * @example
	 * <pre name="test">
	 * #THROWS SailoException
	 * Landlord landlord = new Landlord();
	 * Talo talo1 = new Talo(); Talo talo2 = new Talo();
	 * landlord.getTaloja() === 0;
	 * landlord.lisaa(talo1); landlord.getTaloja() === 1;
	 * landlord.lisaa(talo2); landlord.getTaloja() === 2;
	 * landlord.lisaa(talo2); landlord.getTaloja() === 3;
	 * landlord.annaTalo(0) === talo1;
	 * landlord.annaTalo(1) === talo2;
	 * landlord.annaTalo(2) === talo2;
	 * landlord.annaTalo(1) == talo1 === false;
	 * landlord.annaTalo(1) == talo2 === true;
	 * landlord.annaTalo(3) === talo1; #THROWS IndexOutOfBoundsException
	 * landlord.lisaa(talo1); landlord.getTaloja() === 4;
	 * landlord.lisaa(talo1); landlord.getTaloja() === 5;
	 * landlord.lisaa(talo1); #THROWS SailoException
	 * </pre>
	 */
	public void lisaa(Talo talo) throws SailoException{
		talot.lisaa(talo);
	}
	
	/**
     * Lisää Asunnon tietokantaan.
     * @param asunto asunto, joka halutaan lisätä tietokantaan.
     * @throws SailoException jos on liikaa taloja
     * @example
     * <pre name="test">
     * #THROWS SailoException
     * Landlord landlord = new Landlord();
     * Asunto asunto1 = new Asunto(1); Asunto asunto2 = new Asunto(1);
     * landlord.getAsuntoja() === 0;
     * landlord.lisaa(asunto1); landlord.getAsuntoja() === 1;
     * landlord.lisaa(asunto2); landlord.getAsuntoja() === 2;
     * landlord.lisaa(asunto2); landlord.getAsuntoja() === 3;
     * landlord.annaAsunto(0) === asunto1;
     * landlord.annaAsunto(1) === asunto2;
     * landlord.annaAsunto(2) === asunto2;
     * landlord.annaAsunto(1) == asunto1 === false;
     * landlord.annaAsunto(1) == asunto2 === true;
     * landlord.annaAsunto(3) === asunto1; #THROWS IndexOutOfBoundsException
     * landlord.lisaa(asunto1); landlord.getAsuntoja() === 4;
     * landlord.lisaa(asunto1); landlord.getAsuntoja() === 5;
     * landlord.lisaa(asunto1); #THROWS SailoException
     * </pre>
     */
	public void lisaa(Asunto asunto) throws SailoException{
        asunnot.lisaa(asunto);
    }
	
	/**
	 * Lisää asukkaan tietorakenteeseen.
	 * @param asukas lisättävä asukas
	 * @example
	 * <pre name="test">
     * Landlord landlord = new Landlord();
     * Asukas asukas1 = new Asukas(1); Asukas asukas2 = new Asukas(1);
     * landlord.getAsukkaita() === 0;
     * landlord.lisaa(asukas1); landlord.getAsukkaita() === 1;
     * landlord.lisaa(asukas2); landlord.getAsukkaita() === 2;
     * landlord.lisaa(asukas2); landlord.getAsukkaita() === 3;
     * landlord.annaAsukas(0) === asukas1;
     * landlord.annaAsukas(1) === asukas2;
     * landlord.annaAsukas(2) === asukas2;
     * landlord.annaAsukas(1) == asukas1 === false;
     * landlord.annaAsukas(1) == asukas2 === true;
     * landlord.annaAsukas(3) === asukas1; #THROWS IndexOutOfBoundsException
     * landlord.lisaa(asukas1); landlord.getAsukkaita() === 4;
     * landlord.lisaa(asukas1); landlord.getAsukkaita() === 5;
     * landlord.lisaa(asukas1);
	 * </pre>
	 */
	public void lisaa(Asukas asukas){
	    asukkaat.lisaa(asukas);
	}
	
	/**
	 * Palauttaa listan asukkaista, jotka kuuluvat tiettyyn asuntoon.
	 * @param asunto asunto, jonka asukkaat halutaan
	 * @return lista löydetyistä asukkaista
	 */
	public List<Asukas> annaAsukkaat(Asunto asunto){
	    return asukkaat.annaAsukkaat(asunto.getId());
	}
	
	
	/**
	 * Palauttaa paikassa i olevan Talon viitteen
	 * @param i paikka
	 * @return talo paikassa i
	 * @throws IndexOutOfBoundsException jos i on väärin.
	 */
	public Talo annaTalo(int i) throws IndexOutOfBoundsException {
		return talot.anna(i);
	}
	
	
	/**
     * Palauttaa paikassa i olevan Asunnon viitteen
     * @param i paikka
     * @return Asunto paikassa i
     * @throws IndexOutOfBoundsException jos i on väärin.
     */
	public Asunto annaAsunto(int i) throws IndexOutOfBoundsException {
        return asunnot.anna(i);
    }
	
	/**
	 * Palauttaa paikassa i olevan asukkaan viitteen
	 * @param i paikka
	 * @return asukas paikassa i
	 * @throws IndexOutOfBoundsException jos i on väärin.
	 */
	public Asukas annaAsukas(int i) throws IndexOutOfBoundsException {
		return asukkaat.anna(i);
	}
	
	/**
	 * @param args ei käytössä
	 */
	public static void main(String[] args) {
		Landlord landlord = new Landlord();
		
		Talo talo1 = new Talo();
		Talo talo2 = new Talo();
		
		talo1.rekisteroi();
		talo2.rekisteroi();
		talo1.taytaOletus();
		talo2.taytaOletus();
		
		Asunto asunto1 = new Asunto(talo1.getId());
		asunto1.rekisteroi();
		asunto1.taytaOletus();
		
		Asukas asukas1 = new Asukas(asunto1.getId());
		asukas1.rekisteroi();
		asukas1.taytaOletus();
		
		try {
		landlord.lisaa(talo1);
		landlord.lisaa(talo2);
		landlord.lisaa(asunto1);
		}
		catch(SailoException ex) {
			System.err.println(ex.getMessage());
		}
		landlord.lisaa(asukas1);
		
		System.out.println("=============Landlordin testi=============");
		
		for (int i = 0; i < landlord.getTaloja(); i++) {
			Talo talo = landlord.annaTalo(i);
			System.out.println("Talo paikassa: " + i);
			talo.tulosta(System.out);
		}
		
		for(int i = 0; i < landlord.getAsuntoja(); i++) {
			Asunto asunto = landlord.annaAsunto(i);
			System.out.println("Asunto paikassa: " + i);
			asunto.tulosta(System.out);
		}
		
		for(int i = 0; i < landlord.getAsukkaita(); i++) {
			Asukas asukas = landlord.annaAsukas(i);
			System.out.println("Asukas paikassa: " + i);
			asukas.tulosta(System.out);
		}

	}

}
