package landlordApp;

/**
 * Vastuualueet:
 * -Pitää yllä talojen rekisteriä, osaa lisätä ja poistaa talon
 * -Lukee ja kirjoittaa talot tiedostoon
 * -Osaa etsiä ja lajitella
 * Avustajat: 
 * Talo
 * @author Nico & Eelis
 * @version 12.3.2020
 */
public class Talot {
	
	//Attribuutit
	private int lkm;
	private static final int MAX_LKM = 5;
	private Talo[] talot = new Talo[MAX_LKM];
	
	
	/**
	 * Palauttaa talojen lukumäärän
	 * @return talojen lukumäärä
	 */
	public int getLkm() {
		return lkm;
	}
	

	/**
	 * Lisää tietorakenteeseen uuden talon ja päivittää talojen lukumäärän.
	 * Ottaa talon omistukseensa.
	 * @param talo Lisättävän talon viite
	 * @throws SailoException jos tietorakenne täynnä.
	 * @example
	 * <pre name="test">
	 * #THROWS SailoException
	 * Talot talot = new Talot();
	 * Talo talo1 = new Talo(); Talo talo2 = new Talo();
	 * talot.getLkm() === 0;
	 * talot.lisaa(talo1); talot.getLkm() === 1;
	 * talot.lisaa(talo2); talot.getLkm() === 2;
	 * talot.lisaa(talo2); talot.getLkm() === 3;
	 * talot.anna(0) === talo1;
	 * talot.anna(1) === talo2;
	 * talot.anna(2) === talo2;
	 * talot.anna(1) == talo1 === false;
	 * talot.anna(1) == talo2 === true;
	 * talot.anna(3) === talo1; #THROWS IndexOutOfBoundsException
	 * talot.lisaa(talo1); talot.getLkm() === 4;
	 * talot.lisaa(talo1); talot.getLkm() === 5;
	 * talot.lisaa(talo1); #THROWS SailoException
	 * </pre>
	 */
	public void lisaa(Talo talo) throws SailoException {
		if (lkm >= talot.length) throw new SailoException("Liikaa taloja.");
		talot[lkm] = talo;
		lkm++;
	}
	
	
	/**
	 * Palauttaa talot taulukon paikassa i olevan Talon viitteen
	 * @param i paikka
	 * @return Halutun talon viite
	 * @throws IndexOutOfBoundsException
	 */
	public Talo anna(int i) throws IndexOutOfBoundsException {
		if(i < 0 || lkm <= i)
			throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
		return talot[i];
	}
	
	
	/**
	 * @param args ei käytössä
	 */
	public static void main(String[] args) {
		Talot talot = new Talot();
		Talo talo1 = new Talo(); Talo talo2 = new Talo();
		
		talo1.rekisteroi();
		talo2.rekisteroi();
		talo1.taytaOletus();
		talo2.taytaOletus();
		
		try {
			talot.lisaa(talo1);
			talot.lisaa(talo2);
			talot.lisaa(talo2);
			talot.lisaa(talo2);
			talot.lisaa(talo2);
			talot.lisaa(talo2);
		}
		catch (SailoException ex){
			System.err.println(ex.getMessage());
		}
		
		System.out.println("=============== Talot testi ===============");
		
		for (int i = 0; i < talot.getLkm(); i++) {
			Talo talo = talot.anna(i);
			System.out.println("Talon indeksi: " + i);
			talo.tulosta(System.out);
		}
	}
}
